@@ -1,33 +1,39 @@
 from abc import ABC, abstractmethod
+
 import numpy as np
 import scipy.optimize as sopt
 
 
 class BaseWrapper(ABC):
-
     def get_input(self, input_var):
         self.input_type = type(input_var)
         assert self.input_type in [
-            dict, list, np.ndarray], 'The initial input to your optimized function should be one of dict, list or np.ndarray'
+            dict,
+            list,
+            np.ndarray,
+        ], "The initial input to your optimized function should be one of dict, list or np.ndarray"
         input_, self.shapes = self._concat(input_var)
         self.var_num = input_.shape[0]
         return input_
 
     def get_output(self, output_var):
-        assert 'shapes' in dir(
-            self), 'You must first call get input to define the tensors shapes.'
+        assert "shapes" in dir(
+            self
+        ), "You must first call get input to define the tensors shapes."
         output_var_ = self._unconcat(output_var, self.shapes)
         return output_var_
 
     def get_bounds(self, bounds):
 
         if bounds is not None:
-            if isinstance(bounds, tuple) and not (isinstance(bounds[0], tuple) or isinstance(bounds[0], sopt.Bounds)):
+            if isinstance(bounds, tuple) and not (
+                isinstance(bounds[0], tuple) or isinstance(bounds[0], sopt.Bounds)
+            ):
                 assert len(bounds) == 2
-                new_bounds = [bounds]*self.var_num
+                new_bounds = [bounds] * self.var_num
 
             elif isinstance(bounds, sopt.Bounds):
-                new_bounds = [bounds]*self.var_num
+                new_bounds = [bounds] * self.var_num
 
             elif type(bounds) in [list, tuple, np.ndarray]:
                 if self.input_type in [list, tuple]:
@@ -47,34 +53,37 @@ class BaseWrapper(ABC):
                     if k in bounds.keys():
                         new_bounds += format_bounds(bounds[k], self.shapes[k])
                     else:
-                        new_bounds += [(None, None)
-                                       ]**np.prod(self.shapes[k], dtype=np.int32)
+                        new_bounds += [(None, None)] * np.prod(
+                            self.shapes[k], dtype=np.int32
+                        )
         else:
             new_bounds = bounds
         return new_bounds
 
     def get_constraints(self, constraints, method):
-        if constraints is not None and not isinstance(constraints, sopt.LinearConstraint):
+        if constraints is not None and not isinstance(
+            constraints, sopt.LinearConstraint
+        ):
             assert isinstance(constraints, dict)
-            assert 'fun' in constraints.keys()
-            self.ctr_func = constraints['fun']
-            use_autograd = constraints.get('use_autograd', True)
-            if method in ['trust-constr']:
+            assert "fun" in constraints.keys()
+            self.ctr_func = constraints["fun"]
+            use_autograd = constraints.get("use_autograd", True)
+            if method in ["trust-constr"]:
 
                 constraints = sopt.NonlinearConstraint(
                     self._eval_ctr_func,
-                    lb=constraints.get('lb', -np.inf),
-                    ub=constraints.get('ub', np.inf),
-                    jac=self.get_ctr_jac if use_autograd else '2-point',
-                    keep_feasible=constraints.get('keep_feasible', False),
+                    lb=constraints.get("lb", -np.inf),
+                    ub=constraints.get("ub", np.inf),
+                    jac="2-point",
+                    keep_feasible=constraints.get("keep_feasible", False),
                 )
-            elif method in ['COBYLA', 'SLSQP']:
+            elif method in ["COBYLA", "SLSQP"]:
                 constraints = {
-                    'type': constraints.get('type', 'eq'),
-                    'fun': self._eval_ctr_func,
+                    "type": constraints.get("type", "eq"),
+                    "fun": self._eval_ctr_func,
                 }
                 if use_autograd:
-                    constraints['jac'] = self.get_ctr_jac
+                    constraints["jac"] = self.get_ctr_jac
             else:
                 raise NotImplementedError
         elif constraints is None:
@@ -153,10 +162,10 @@ class BaseWrapper(ABC):
         if isinstance(shapes, dict):
             ten_vals = {}
             for k, sh in shapes.items():
-                next_ind = current_ind+np.prod(sh, dtype=np.int32)
+                next_ind = current_ind + np.prod(sh, dtype=np.int32)
                 ten_vals[k] = self._reshape(
-                    self._gather(ten, current_ind, next_ind), sh)
-
+                    self._gather(ten, current_ind, next_ind), sh
+                )
                 current_ind = next_ind
 
         elif isinstance(shapes, list) or isinstance(shapes, tuple):
@@ -165,9 +174,10 @@ class BaseWrapper(ABC):
             else:
                 ten_vals = []
                 for sh in shapes:
-                    next_ind = current_ind+np.prod(sh, dtype=np.int32)
+                    next_ind = current_ind + np.prod(sh, dtype=np.int32)
                     ten_vals.append(
-                        self._reshape(self._gather(ten, current_ind, next_ind), sh))
+                        self._reshape(self._gather(ten, current_ind, next_ind), sh)
+                    )
 
                     current_ind = next_ind
 
@@ -192,9 +202,9 @@ class BaseWrapper(ABC):
 def format_bounds(bounds_, sh):
     if isinstance(bounds_, tuple):
         assert len(bounds_) == 2
-        return [bounds_]*np.prod(sh, dtype=np.int32)
+        return [bounds_] * np.prod(sh, dtype=np.int32)
     elif isinstance(bounds_, sopt.Bounds):
-        return [bounds_]*np.prod(sh, dtype=np.int32)
+        return [bounds_] * np.prod(sh, dtype=np.int32)
     elif isinstance(bounds_, list):
         assert np.prod(sh) == len(bounds_)
         return bounds_
